<?php

namespace App\Http\Controllers\Api\ShaheenApis;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Illuminate\Support\Facades\Log;

use App\Models\Request as AppRequest;
use App\Models\Runsheet;

class ShaheenApisController extends Controller
{
    public $key = "shaheen-api090078601";

    public function createReference(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {
            $refExists = DB::table('collection')->where('co_id', $request->id)->first();

            if (!$refExists) {
                DB::table('collection')->insert([
                    "co_id" => $request->id,
                    "store_id" => $request->store_username,
                    'branchid' => $request->branch_id,
                    // "periodid"=>$request->period_id,
                    "create_userid" => $request->user_id,
                    "wallayah_id" => $request->wilaya_id,
                    "region_id" => $request->area_id,
                    "co_quantity" => $request->quantity,
                    "co_date" => $request->collection_date,
                    "collected_cash" => $request->collected_cash,
                ]);


                return response()->json([
                    "status" => "success",
                    "return" => false,
                    "message" => "Collection created in old system"
                ]);
            } else {
                $collection_id =  DB::table('collection')->insertGetId([
                    "store_id" => $request->store_username,
                    'branchid' => $request->branch_id,
                    // "periodid"=>$request->period_id,
                    "create_userid" => $request->user_id,
                    "wallayah_id" => $request->wilaya_id,
                    "region_id" => $request->area_id,
                    "co_quantity" => $request->quantity,
                    "co_date" => $request->collection_date,
                    "collected_cash" => $request->collected_cash,
                ]);


                return response()->json([
                    "status" => "success",
                    "return" => true,
                    "collection_id" => $collection_id,
                    "message" => "Collection created in old system"
                ]);
            }
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "return" => false,
                "message" => $e->getMessage()
            ]);
        }
    }


    public function createReference2(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $collection_id =  DB::table('collection')->insertGetId([
                "store_id" => $request->store_username,
                'branchid' => $request->branch_id,
                // "periodid"=>$request->period_id,
                "create_userid" => $request->user_id,
                "wallayah_id" => $request->wilaya_id ? $request->wilaya_id : 0,
                "region_id" => $request->area_id ? $request->area_id : 0,
                "co_quantity" => $request->quantity,
                "co_date" => $request->collection_date,
                "collected_cash" => $request->collected_cash,
                "is_completed" => "1",
                "driver_id" => $request->driver_username,
                "co_date" => date("Y-m-d"),
                "co_time" => date("H:i:s"),
                "co_notes" => $request->notes,
                "lat" => $request->lat,
                "lng" => $request->lng,
                "is_sync" => "1",

            ]);




            return response()->json([
                "status" => "success",
                "return" => true,
                "collection_id" => $collection_id,
                "message" => "Collection created in old system"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "return" => false,
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ]);
        }
    }


    public function createStore(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }


        try {
            $exists = DB::table('login')->where('s_username', $request->username)->first();

            if (!$exists) {
                DB::table('login')->insert([
                    "store_code" => $request->store_code,
                    'branchid' => $request->branch_id,
                    's_name' => $request->name,
                    's_mobile' => $request->mobile,
                    's_password' => bcrypt($request->password),
                    's_type' => "store",
                    's_email' => $request->email,
                    's_username' => $request->username,
                    'created_date' => date('Y-m-d H:i:s'),
                ]);


                return response()->json([
                    "status" => "success",
                    "message" => "Store created in old system"
                ]);
            } else {
                return response()->json([
                    "status" => "error",
                    "message" => "Store already exists in old system"
                ]);
            }
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function createDriver(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }


        try {
            $exists = DB::table('login')->where('s_username', $request->username)->first();

            if (!$exists) {
                DB::table('login')->insert([
                    'branchid' => $request->branch_id,
                    's_name' => $request->name,
                    's_mobile' => $request->mobile,
                    's_password' => bcrypt($request->password),
                    's_type' => "driver",
                    's_email' => $request->email,
                    's_username' => $request->username,
                    'created_date' => date('Y-m-d H:i:s'),
                ]);


                return response()->json([
                    "status" => "success",
                    "message" => "Driver created in old system"
                ]);
            } else {
                return response()->json([
                    "status" => "error",
                    "message" => "Driver already exists in old system"
                ]);
            }
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function assignDriverToCollection(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }


        try {
            $collection = DB::table('collection')->where('co_id', $request->collection_id)->first();

            if (!$collection) {
                return response()->json([
                    "status" => "error",
                    "message" => "Collection not found"
                ]);
            }

            $driver = DB::table('login')->where('s_username', $request->driver_username)->first();

            if (!$driver) {
                return response()->json([
                    "status" => "error",
                    "message" => "Driver not found"
                ]);
            }

            DB::table('collection')->where('co_id', $request->collection_id)->update(['driver_id' => $driver->s_id]);


            return response()->json([
                "status" => "success",
                "message" => "Driver assigned to collection",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }



    public function updateShipperComments(Request $request){


            if(isset($request->problem_id)){

                if(isset($request->trader_comments)){

                    DB::table('order_problems')->where('id', $request->problem_id)->update(['trader_comments' => $request->trader_comments]);    
                }
                else{

                    return response()->json([
                        "status" => "error",
                        "message" =>'comments is  missing'
                    ]);

                }

                

            return response()->json([
                "status" => "success",
                "message" => "Problem is comments updated successfully",
            ]);


            }
            else{

            return response()->json([
                "status" => "error",
                "message" =>'Id is missing'
            ]);

            }


         

    }

    public function logChange($cpid, $parent_type, $parentid, $data = [], $module = '', $userid = 0, $branchid = 0)
    {
        $userid = $userid > 0 ? $userid : $this->userid;
        $branchid = $branchid > 0 ? $branchid : $this->user_branchid;

        $fields = [
            'cpid' => $cpid,
            'parent_type' => $parent_type,
            'branchid' => $branchid,
            'parentid' => $parentid,
            'userid' => $userid,
            'module' => $module,
            'data' => json_encode($data),
        ];

        DB::table("change_log")->insert($fields);
    }


    public function spearateOrderIds($order_ids)
    {
        $order = null;
        $preInvoice = null;

        $a_ids = [];
        $p_ids = [];

        foreach ($order_ids as $order_id) {

            $order_id = strtolower($order_id);

            if (strpos($order_id, "p") !== false) {

                $order_id = trim($order_id, "p");

                $p_ids[] = $order_id;
            } else if (strpos($order_id, "a") !== false) {
                $order_id = trim($order_id, "a");

                $a_ids[] = $order_id;
            } else {
                $a_ids[] = $order_id;
            }
        }




        return [
            "a_ids" => $a_ids,
            "p_ids" => $p_ids,
        ];
    }


    public function createRunsheet(Request $request)
    {




        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {


            $driver = DB::table('login')->where('s_username', $request->driver_username)->first();



            $rs_id = DB::table('run_sheet')->insertGetId([
                'branchid' => $request->branch_id,
                "agentid" => $driver->s_id,
                "create_userid" => $request->user_id,
                "is_sync" => 1
            ]);


            $order_ids = $request->orders;
            $sperated_ids = $this->spearateOrderIds($order_ids);

            $a_ids = $sperated_ids['a_ids'];
            $p_ids = $sperated_ids['p_ids'];


            DB::table("requests as rq")
                ->leftJoin("pre_invoice_map as pim", "pim.actual_id", "rq.r_id")
                ->where("r_status", "!=", "completed")
                ->where("r_status", "!=", "canceled")
                ->where("r_status", "!=", "return")
                ->where("r_status", "!=", "RTO")
                ->where(function ($q) use ($a_ids, $p_ids) {
                    $q->whereIn("rq.r_id", $a_ids)
                        ->orWhereIn("pim.pid", $p_ids)
                        ->orWhereIn("rq.external_way_bill_number", $a_ids);
                })
                ->update([
                    "rq.r_status" => "OFD",
                    "rq.rsid" => $rs_id,
                    "rq.operation_status" => NULL,
                    "delivery_attempts"=>DB::raw("rq.delivery_attempts+1")
                ]);


            $backOrders = DB::table("requests as rq")
                ->leftJoin("pre_invoice_map as pm", "rq.r_id", "=", "pm.actual_id")
                ->where("rq.rsid", $rs_id)
                ->get();

            foreach ($backOrders as $order) {

                $this->logChange(37, 1, $order->r_id, [
                    'rsid' => $rs_id,
                    'driverid' => $driver->s_id
                ], 'runsheet', 1069, 1);
            }

            return response()->json([
                "status" => "success",
                "run_sheet_id" => $rs_id,
                "orders" => $backOrders,
                "message" => "Runsheet created"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString()
            ]);
        }
    }


    public function createSendsheet(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {




            $transfer_id = DB::table('transfer')->insertGetId([
                'branchid' => $request->from_branch_id,
                "userid" => 1069,
                "receive_branchid" => $request->to_branch_id,
                "transfer_status" => 1
            ]);

            $items_id = [];

            $r_ids = [];


            if ($request->order_ids) {
                foreach ($request->order_ids as $order_id) {

                    $order_id = strtolower($order_id);

                    if (strpos($order_id, "p") !== false) {

                        $order_id = trim($order_id, "p");

                        $preInvoice = DB::table("pre_invoice_map")->where("pid", $order_id)->first();

                        if ($preInvoice) {
                            if ($preInvoice->actual_id) {
                                $r_ids[] = $preInvoice->actual_id;
                            }
                        }
                    } else if (strpos($order_id, "a") !== false) {
                        $order_id = trim($order_id, "a");

                        $req =  DB::table('requests')->where('r_id', $order_id)->first();

                        if ($req) {
                            $r_ids[] = $req->r_id;
                        }
                    } else {
                        $req = DB::table('requests')->where('external_way_bill_number', $order_id)->first();
                        if ($req) {
                            $r_ids[] = $req->r_id;
                        }
                    }
                }
            }


            //  return response()->json([
            //     "status" => "success",
            //    "request" => $r_ids
            // ]);


            if (!empty($r_ids)) {
                foreach ($r_ids as $r_id) {

                    $requestExistinSendSheet = DB::table("transfer_items")->where("r_id", $r_id)->whereNull("receiver_user_id")->first();

                    if (!$requestExistinSendSheet) {
                        $transfer_item_id = DB::table("transfer_items")
                            ->insertGetId([
                                "r_id" => $r_id,
                                "transferid" => $transfer_id,
                                "sender_user_id" => 1069,
                                "sender_branch_id" => $request->from_branch_id,
                                "sent_at" => now(),
                                "status" => 1
                            ]);

                        $items_id[] = $transfer_item_id;


                        $this->logChange(40, 1, $r_id, ['to_branch' => $request->to_branch_id, 'transfer_id' => $transfer_id], 'send_sheet', 1069, 1);
                    }
                }
            }


            $requests = DB::table("requests as rq")
                ->leftJoin("pre_invoice_map as pm", "rq.r_id", "=", "pm.actual_id")
                ->whereIn("rq.r_id", $r_ids)
                ->get();



            return response()->json([
                "status" => "success",
                "send_sheet_id" => $transfer_id,
                "send_sheet_items_ids" => $items_id,
                "requests" => $requests,
                "message" => "Sendsheet created"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }



    public function createReceivesheet(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {


            $r_ids = [];

            $items_id = [];

            if ($request->orders) {
                foreach ($request->orders as $order_id) {

                    $order_id = strtolower($order_id);

                    if (strpos($order_id, "p") !== false) {

                        $order_id = trim($order_id, "p");

                        $preInvoice = DB::table("pre_invoice_map")->where("pid", $order_id)->first();

                        if ($preInvoice) {
                            if ($preInvoice->actual_id) {
                                $r_ids[] = $preInvoice->actual_id;
                            }
                        }
                    } else if (strpos($order_id, "a") !== false) {
                        $order_id = trim($order_id, "a");

                        $req =  DB::table('requests')->where('r_id', $order_id)->first();

                        if ($req) {
                            $r_ids[] = $req->r_id;
                        }
                    } else {
                        $req = DB::table('requests')->where('external_way_bill_number', $order_id)->first();
                        if ($req) {
                            $r_ids[] = $req->r_id;
                        }
                    }
                }
            }





            if (!empty($r_ids)) {
                foreach ($r_ids as $r_id) {


                    DB::table("transfer_items")->where("r_id", $r_id)
                        ->update([
                            "receiver_user_id" => 1069,
                            "receiver_branch_id" => $request->branch_id,
                            "received_at" => now(),
                            "status" => 3,
                        ]);




                    $transfer_item = DB::table("transfer_items")->where("r_id", $r_id)->first();

                    if ($transfer_item) {

                        $this->logChange(41, 1, $r_id, [], 'receive_sheet', 1069, $request->branch_id);


                        $totalSendSheetItems = DB::table("transfer_items")->where("transferid", $transfer_item->transferid)->count();

                        $totalReceivedItems = DB::table('transfer_items')->whereNotNull("receiver_user_id")->where("transferid", $transfer_item->transferid)->count();

                        if ($totalSendSheetItems == $totalReceivedItems) {

                            DB::table("transfer")->where("transferid", $transfer_item->transferid)->update(["receive_userid" => 1069, "transfer_status" => 3]);
                        }


                        $items_id[] = $transfer_item->id;
                    }
                }
            }
            // 


            return response()->json([
                "status" => "success",
                "send_sheet_items_ids" => $items_id,
                "message" => "Sendsheet created"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString()
            ]);
        }
    }




    public function LogSheetOrderConfirm(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {
            foreach ($request->requests as $order) {
                DB::table("requests")
                    ->where("r_id", $order["r_id"])
                    ->where("co_id", $order["co_id"])
                    ->update([
                        "pickup_confirm" => 1,
                        "pickup_confirm_date" => date('Y-m-d H:i:s')
                    ]);
            }


            return response()->json([
                "status" => "success",
                "message" => "Order pickup confirmed in old system"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }




    public function dataEntryLogsheet(Request $request)
    {


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {
            $data = [
                "is_data_entry" => 1,
            ];

            if (isset($request->name) && $request->name != "") {
                $data["r_name"] = $request->name;
            }

            if (isset($request->phone) && $request->phone != "") {
                $data["r_phone"] = $request->phone;
            }

            if (isset($request->price) && $request->price != "") {
                $data["r_price"] = $request->price;
            }

            if (isset($request->ship_price) && $request->ship_price != "") {
                $data["r_ship_price"] = $request->ship_price;
            }


            if (isset($request->cc) && $request->cc != "") {
                $data["cc"] = $request->cc;
            }

            if (isset($request->delivery_type) && $request->delivery_type != "") {
                $data["r_delivery_type"] = $request->delivery_type;
            }

            if (isset($request->wilaya_id) && $request->wilaya_id != "") {
                $data["wilaya_id"] = $request->wilaya_id;
                $data["r_wallayah"] = $request->wilaya_id;
            }

            if (isset($request->area_id) && $request->area_id != "") {
                $data["region_id"] = $request->area_id;
                $data["r_region_id"] = $request->area_id;
                $data["r_region"] = $request->area_id;
            }

            if (isset($request->address) && $request->address != "") {
                $data["address"] = $request->address;
            }

            if (isset($request->not_clear) && $request->not_clear != "") {
                $data["not_clear"] = $request->not_clear;
            }

            if (isset($request->cop_cc_not_match) && $request->cop_cc_not_match != "") {
                $data["cop_cc_not_match"] = $request->cop_cc_not_match;
            }

            // 
            if (isset($request->external_way_bill_number) && $request->external_way_bill_number != "") {
                $data["external_way_bill_number"] = $request->external_way_bill_number;
            }

            DB::table('requests')->where("r_id", $request->r_id)->update($data);

            DB::table("change_log")->insert([
                "cpid" => 11,
                "parent_type" => 1,
                "parentid" => $request->r_id,
                "module" => "logSheet",
                "data" => json_encode($data),
                "userid" => 1069,
            ]);



            DB::table('order_log')->insert([
                "order_id" => $request->r_id,
                "external_text" => "Logsheet data entry from shaheen api",
                "internal_text" => "Logsheet data entry from shaheen api",
                "short_text" => "Logsheet data entry from shaheen api",
                "status_id" => 11
            ]);

            DB::table("data_entry_collection_orders")->insert([
                "collection_id" => $request->co_id,
                "order_id" => $request->r_id,
                "status" => 1,
                "user_id" => 1069
            ]);

            return response()->json([
                "status" => "success",
                "message" => "New data entry done of order {$request->r_id}",
                "request" => $request->all()
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "request" => $request->all(),
                "trace" => $e->getTraceAsString(),
            ]);
        }
    }

    public function updateCOP(Request $request)
    {
        // return response()->json($request->all(),);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            DB::table('collection')->where("co_id", $request->co_id)->update([
                "collected_cash" => $request->cop
            ]);


            return response()->json([
                "status" => "success",
                "message" => $request->co_id . " COP updated"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "request" => $request->all(),
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString()
            ]);
        }
    }



    public function debreifingChangeOrderStatus(Request $request)
    {
        // return response()->json($request->all(),);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            DB::table('requests')->where("r_id", $request->order_id)->update([
                "r_status" => $request->status,
                "r_status_reason" => $request->status_reason,
                "driver_id" => NULL,
                "rsid" => 0,
            ]);

            $order = DB::table('requests')->where("r_id", $request->order_id)->first();

            $driver = DB::table("login")->where("s_username", $order->driver_id)->first();

            if ($order->r_status == "F") {

                $this->logChange(30, 1, $order->r_id, ['driverid' => $driver->s_id, 'rsid' => $order->rsid], 'debriefing', 1069, 1);
            } else if ($order->r_status == "BR") {

                $this->logChange(35, 1, $order->r_id, ['driverid' => $driver->s_id, 'rsid' => $order->rsid], 'debriefing', 1069, 1);
            }

            return response()->json([
                "status" => "success",
                "message" => "debreifing order {$request->order_id} status updated to {$request->status} in old system"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "request" => $request->all(),
                "trace" => $e->getTraceAsString(),
            ]);
        }
    }




    public function debreifingConfirmRunsheets(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $driver_username = $request->driver_username;

            $driver = DB::table("login")->where("s_username", $driver_username)->first();


            $voucher_id = DB::table("cashier_voucher")->insertGetId([
                "voucher_amount" => 0,
                "branch_id" => 1,
                "type" => "runsheet",
                "created_by" => "shaheen_api",
                "driver_id" => $driver->s_id
            ]);


            DB::table('run_sheet')->whereIn("rsid", $request->runsheets)->update([
                "is_confirmed" => 1,
                'is_confirmed_new' => 1,
                "is_confirmed_from_dalilee" => 0
            ]);


            foreach ($request->runsheets as $rs_id) {

                $this->logChange(43, 1, $rs_id, ['runsheet' => $rs_id, 'user' => 1069], 'debriefing', 1069, 1);
            }

            DB::table('requests')->where("r_status", "completed")->where("driver_id", $driver_username)->whereIn("rsid", $request->runsheets)->update([
                "runsheet_confirm" => 1,
                "runsheet_confirm_date" => date('Y-m-d H:i:s')
            ]);

            $orders = DB::table('requests')->where("r_status", "completed")->where("driver_id", $driver_username)->whereIn("rsid", $request->runsheets)->get();

            foreach ($orders as $order) {

                $this->logChange(43, 1, $order->r_id, ['runsheet' => $order->rsid, 'user' => 1069], 'debriefing', 1069, 1);
            }

            // 

            return response()->json([
                "status" => "success",
                "voucher_id" => $voucher_id,
                "message" => "Runsheet confirmed in old system"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString()
            ]);
        }
    }


    public function LogSheetConfirm(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {



            $driver_username = $request->driver_username;
            $driver = DB::table("login")->where("s_username", $driver_username)->first();


            $totalCOP = 0;


            foreach ($request->collections as $co_id) {

                $collection = DB::table("collection")
                    ->where("co_id", $co_id)->first();

                if ($collection) {
                    $totalCOP += floatval($collection->collected_cash);


                    $this->logChange(44, 3, $collection->co_id, [
                        'logsheet' => $collection->co_id,
                        'user' => "Shaheen Api"
                    ], 'logsheetclearance', 1069, 1);
                }
            }



            $voucher_id = DB::table("cashier_voucher")->insertGetId([
                "voucher_amount" => $totalCOP,
                "branch_id" => 1,
                "type" => "logsheet",
                "created_by" => "shaheen_api",
                "driver_id" => $driver->s_id
            ]);


            DB::table("collection")
                ->whereIn("co_id", $request->collections)->update([
                    "is_confirmed" => 1,
                    "is_confirmed_from_dalilee" => 0
                ]);




            return response()->json([
                "status" => "success",
                "voucher_id" => $voucher_id,
                "message" => "Collections confirmed in old system"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ]);
        }
    }

    public function cashierClose(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }


        try {

            $type = $request->type == "log_sheet" ? "logsheet" : "runsheet";


            $dataFromShaheen = $request->data;


            foreach ($dataFromShaheen as $data) {

                $voucher_id = $data["voucher_id"];
                $cahsierNote = $data["note"];

                $voucher = DB::table("cashier_voucher")->where("id", $voucher_id)->first();

                $driver_id = $voucher->driver_id;

                $driver = DB::table("login")->where("s_id", $driver_id)->first();

                $driver_username = $driver->s_username;

                if ($type == "logsheet") {

                    $collection_ids = $data["cashier_collections"];

                    $totalComission = 0;
                    $totalOrders = 0;

                    $totalCOP = 0;




                    DB::table("collection")
                        ->whereIn("co_id", $collection_ids)
                        ->update([
                            "is_paid" => 1
                        ]);


                    DB::table('requests')->whereIn("co_id", $collection_ids)->update([
                        "cashier_confirm" => 1,
                        "is_paid" => 1,
                        "is_paid_cc" => 1,
                        "cashier_confirm_date" => date("Y-m-d H:i:s")
                    ]);



                    foreach ($collection_ids as $co_id) {

                        $collection = DB::table("collection")
                            ->where("co_id", $co_id)->first();

                        if ($collection) {
                            $totalCOP += floatval($collection->collected_cash);
                        }

                        $requests = DB::table("requests")->where("co_id", $co_id)->count();

                        $orderInSheet = 0;
                        $pershipment = 0;
                        $perCollectionAmount = 0;

                        if ($requests > 0) {


                            $orderInSheet = $requests;
                            $totalOrders += $orderInSheet;

                            $pickup_range = DB::table("pickup_range")->where("start_no_orders", ">=", $orderInSheet)->where("end_no_orders", "<=", $orderInSheet)->orderBy("id", "desc")->first();


                            if ($pickup_range) {

                                $package_id = $pickup_range->id;

                                $driver_package = DB::table("driver_pickup_comission")->where("pickup_range_id", $package_id)->where("driver_id", $driver_id)->first();


                                if ($orderInSheet > 1  && $orderInSheet <= 9) {

                                    $pershipment = 1 / $orderInSheet;
                                } else {

                                    $pershipment = $driver_package ? $driver_package->per_shipment_amount : 0;
                                }

                                $perCollectionAmount = $orderInSheet * $pershipment;

                                $totalComission += $perCollectionAmount;
                            }
                        }

                        DB::table("cashier_collection")->insertGetId([
                            "voucher_id" => $voucher_id,
                            "receiver_id" => "shaheen_api",
                            "branch_id" => 1,
                            "runsheet_id" => $collection->co_id,
                            "runsheet_amount" => $collection->collected_cash,
                            "received_amount" => $collection->collected_cash,
                            "no_orders" => $orderInSheet,
                            "commision_rate" => $pershipment,
                            "commision_amount" => $perCollectionAmount,
                            "driver_id" => $driver_id
                        ]);


                        $this->logChange(45, 3, $co_id, ['referncetype' => 'loghsheet', 'referenceid' => $co_id, 'driver' => $driver_username, 'user' => 1069], "cashierview", 1069, 1);
                    }

                    DB::table("cashier_voucher")->where("id", $voucher_id)->update([
                        "voucher_amount" => $totalCOP,
                        "no_orders" => $totalOrders,
                        "note" => $cahsierNote,
                        "commision_amount" => $totalComission
                    ]);
                } else {
                    $runsheet_ids = $data["cashier_collections"];

                    DB::table("run_sheet")
                        ->whereIn("rsid", $runsheet_ids)
                        ->update([
                            "is_paid" => 1
                        ]);


                    DB::table('requests')->whereIn("rsid", $runsheet_ids)->update([
                        "cashier_confirm" => 1,
                        "is_paid" => 1,
                        "is_paid_cc" => 1,
                        "cashier_confirm_date" => date("Y-m-d H:i:s")
                    ]);

                    $totalCOP = 0;
                    $totalOrders = 0;
                    $totalComission = 0;

                    foreach ($runsheet_ids as $rsid) {

                        $requests = DB::table("requests")->where("r_status", "completed")->where("driver_id", $driver_username)->where("rsid", $rsid)->count();

                        $run_sheet_amount = 0;
                        $orderInSheet = 0;
                        $pershipment = 0;

                        $perCollectionAmount = 0;


                        $this->logChange(45, 3, $rsid, ['referncetype' => 'runsheet', 'referenceid' => $rsid, 'driver' => $driver_username, 'user' => 1069], "cashierview", 1069, 1);



                        if ($requests > 0) {

                            $orders = DB::table("requests")->where("r_status", "completed")->where("driver_id", $driver_username)->where("rsid", $rsid)->get();


                            foreach ($orders as $order) {

                                $totalCOP += $order->r_price;

                                $run_sheet_amount += $order->r_price;
                            }


                            $orderInSheet = $requests;
                            $totalOrders += $orderInSheet;

                            $pickup_range = DB::table("pickup_range")->where("start_no_orders", ">=", $orderInSheet)->where("end_no_orders", "<=", $orderInSheet)->orderBy("id", "desc")->first();


                            if ($pickup_range) {

                                $package_id = $pickup_range->id;

                                $driver_package = DB::table("driver_pickup_comission")->where("pickup_range_id", $package_id)->where("driver_id", $driver->s_id)->first();


                                if ($orderInSheet > 1  && $orderInSheet <= 9) {

                                    $pershipment = 1 / $orderInSheet;
                                } else {

                                    $pershipment = $driver_package ? $driver_package->per_shipment_amount : 0;
                                }

                                $perCollectionAmount = $orderInSheet * $pershipment;

                                $totalComission += $perCollectionAmount;
                            }
                        }


                        $cash =  DB::table("cashier_collection")->insertGetId([
                            "voucher_id" => $voucher_id,
                            "receiver_id" => "shaheen_api",
                            "branch_id" => 1,
                            "runsheet_id" => $rsid,
                            "runsheet_amount" => $run_sheet_amount,
                            "received_amount" => $run_sheet_amount,
                            "no_orders" => $orderInSheet,
                            "commision_rate" => $pershipment,
                            "commision_amount" => $perCollectionAmount,
                            "driver_id" => $driver_id
                        ]);

                        // return response()->json([
                        //     "res" => $cash,
                        // ]);

                    }

                    DB::table("cashier_voucher")->where("id", $voucher_id)->update([
                        "voucher_amount" => $totalCOP,
                        "no_orders" => $totalOrders,
                        "note" => $cahsierNote,
                        "commision_amount" => $totalComission
                    ]);
                }

                // 
                $bank_image = $data["image_name"];

                if ($bank_image != "no_image") {

                    $is_cashier_confirm = DB::table('cashier_confirm')->where("bank_image", $bank_image)->whereNull("voucher_id")->first();

                    if ($is_cashier_confirm) {

                        DB::table('cashier_confirm')->where("bank_image", $bank_image)->update([
                            "status" => 1,
                            "voucher_id" => $voucher_id
                        ]);
                    } else {
                        DB::table('cashier_confirm')->insert([
                            "driver_id" => $driver->s_id,
                            "status" => 1,
                            "voucher_id" => $voucher_id,
                            "bank_image" => $bank_image,
                            "closing_date" => date("Y-m-d"),
                            "closing_type" => $type
                        ]);
                    }
                }
            }


            return response()->json([
                "status" => "success",
                "request" => $request->all(),
                "message" => "cashier cleared"
            ]);
        } catch (\Exception $e) {

            Log::info("Error in CashierClear:  " . $e->getMessage());


            return response()->json([
                "status" => "error",
                "request" => $request->all(),
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString()
            ]);
        }
    }


    public function cashierCloseOld(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        $return = [
            "voucher_ids" => [],
            "cashier_collection_ids" => []
        ];


        try {
            foreach ($request->vouchers as $voucher) {



                // return response()->json($voucher);

                $voucherExists = DB::table('cashier_voucher')->where("id", $voucher["voucher"]["id"])->first();

                $voucher_driver = DB::table('login')->where("s_username", $voucher["voucher"]["driver_username"])->first();



                if ($voucherExists) {
                    $voucher_id = DB::table('cashier_voucher')->insertGetId([
                        "voucher_number" => $voucher["voucher"]["voucher_number"],
                        "voucher_amount" => $voucher["voucher"]["voucher_amount"],
                        "branch_id" => $voucher["voucher"]["branch_id"],
                        "no_orders" => $voucher["voucher"]["no_orders"],
                        "created_by" => $voucher["voucher"]["created_by"],
                        "driver_id" => $voucher_driver->s_id,
                        "type" => $voucher["voucher"]["type"],
                        "commision_amount" => $voucher["voucher"]["commission_amount"],
                        "is_transferd" => $voucher["voucher"]["is_transferd"],
                        // "note"=>$voucher["voucher"]["note"],
                    ]);

                    $return["voucher_ids"][] = [
                        "old_id" => $voucher["voucher"]["id"],
                        "new_id" => $voucher_id
                    ];
                } else {
                    DB::table('cashier_voucher')->insertGetId([
                        "id" => $voucher["voucher"]["id"],
                        "voucher_number" => $voucher["voucher"]["voucher_number"],
                        "voucher_amount" => $voucher["voucher"]["voucher_amount"],
                        "branch_id" => $voucher["voucher"]["branch_id"],
                        "no_orders" => $voucher["voucher"]["no_orders"],
                        "created_by" => $voucher["voucher"]["created_by"],
                        "driver_id" => $voucher_driver->s_id,
                        "type" => $voucher["voucher"]["type"],
                        "commision_amount" => $voucher["voucher"]["commission_amount"],
                        "is_transferd" => $voucher["voucher"]["is_transferd"],
                        // "note"=>$voucher["voucher"]["note"],
                    ]);

                    $voucher_id = $voucher["voucher"]["id"];
                }



                if ($voucher["cashierCollections"] !== 0) {
                    foreach ($voucher["cashierCollections"] as $cashierCollection) {
                        $cashierCollectionExists = DB::table('cashier_collection')->where("id", $cashierCollection["id"])->first();

                        $cashier_collection_driver = DB::table('login')->where("s_username", $cashierCollection["driver_username"])->first();


                        if ($cashierCollectionExists) {
                            $cashier_collection_id =    DB::table('cashier_collection')->insertGetId([
                                "voucher_id" => $voucher_id,
                                "branch_id" => $cashierCollection["branch_id"],
                                "runsheet_id" => $cashierCollection["runsheet_id"] ? $cashierCollection["runsheet_id"] : $cashierCollection["collection_id"],
                                "runsheet_amount" => $cashierCollection["collection_amount"],
                                "no_orders" => $cashierCollection["no_orders"],
                                "commision_amount" => $cashierCollection["commission_amount"],
                                "commision_rate" => $cashierCollection["commission_rate"],
                                "driver_id" => $cashier_collection_driver->s_id,
                                // "comments"=>$cashierCollection["comments"],
                                "receiver_id" => 1069,
                                "received_amount" => $cashierCollection["commission_amount"],

                            ]);

                            $return["cashier_collection_ids"][] = [
                                "old_id" => $cashierCollection["id"],
                                "new_id" => $cashier_collection_id
                            ];
                        } else {
                            DB::table('cashier_collection')->insertGetId([
                                "id" => $cashierCollection["id"],
                                "voucher_id" => $voucher_id,
                                "branch_id" => $cashierCollection["branch_id"],
                                "runsheet_id" => $cashierCollection["runsheet_id"] ? $cashierCollection["runsheet_id"] : $cashierCollection["collection_id"],
                                "runsheet_amount" => $cashierCollection["collection_amount"],
                                "no_orders" => $cashierCollection["no_orders"],
                                "commision_amount" => $cashierCollection["commission_amount"],
                                "received_amount" => $cashierCollection["commission_amount"],
                                "commision_rate" => $cashierCollection["commission_rate"],
                                "driver_id" => $cashier_collection_driver->s_id,
                                "receiver_id" => 1069,

                            ]);
                        }


                        if ($cashierCollection["type"] == "runsheet") {
                            DB::table('run_sheet')->where("rsid", $cashierCollection["runsheet_id"])->update([
                                "is_paid" => 1
                            ]);


                            DB::table('requests')->where("rsid", $cashierCollection["runsheet_id"])->update([
                                "cashier_confirm" => 1,
                                "cashier_confirm_date" => date("Y-m-d H:i:s")
                            ]);
                        } elseif ($cashierCollection["type"] == "collection") {
                            DB::table('collection')->where("co_id", $cashierCollection["collection_id"])->update([
                                "is_paid" => 1
                            ]);

                            DB::table('requests')->where("co_id", $cashierCollection["collection_id"])->update([
                                "is_paid_cc" => 1,
                                "is_paid_cc_date" => date("Y-m-d H:i:s")
                            ]);
                        }
                    }
                }


                if ($voucher["cashierConfirm"] !== 0) {
                    DB::table('cashier_confirm')->where("voucher_id", $voucher_id)->update([
                        "status" => 1
                    ]);
                }
            }




            return response()->json([
                "status" => "success",
                "return" => $return,
                "message" => "cashier cleared"
            ]);
        } catch (\Exception $e) {

            Log::info("Error in CashierClear:  " . $e->getMessage());


            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }



    public function dataEntryNextOrder(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            DB::table('requests')->where("r_id", $request->r_id)->update([

                "r_phone" => $request->phone,
                "r_name" => $request->name,
                "r_price" => $request->price,
                "r_delivery_type" => $request->delivery_type,
                "wilaya_id" => $request->wilaya_id,
                "region_id" => $request->area_id,
                "r_ship_price" => $request->ship_price,
                "cc" => $request->cc,
                "notes" => $request->notes,
                "not_clear" => $request->not_clear,


            ]);

            $dataEntryCollectionOrdersExists = DB::table('data_entry_collection_orders')->where("order_id", $request->r_id)->first();

            if ($dataEntryCollectionOrdersExists) {
                DB::table("data_entry_collection_orders")->update([
                    "collection_id" => $request->collection_id,
                    "order_id" => $request->r_id,
                    "user_id" => $request->user_id,
                    "status" => 1
                ]);
            } else {
                DB::table("data_entry_collection_orders")->insert([
                    "collection_id" => $request->collection_id,
                    "order_id" => $request->r_id,
                    "user_id" => $request->user_id,
                    "status" => 1
                ]);
            }

            return response()->json([
                "status" => "sucess",
                "message" => "successfully updated"
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }

    // 

    public function dataEntryUpdateCollection(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            DB::table('collection')->where("co_id", $request->co_id)->update([

                "dataentry_status" => 1,

            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }

    public function getLastPid(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $temp = DB::table('pre_temp_table')->orderBy('id', 'desc')->first();

            $totalPidsToGenerate = $request->totalPidsToGenerate;

            $lastPid = $temp->temp_pid;


            $store = DB::table("login")->where("store_code", $request->store_code)->first();


            for ($i = 0; $i < $totalPidsToGenerate; $i++) {
                $lastPid++;

                $dataToInsert[] = [
                    "pid" => $lastPid,
                    "create_userid" => $request->user_id,
                    "storeid" => $store->s_id,
                ];
            }

            DB::table('pre_invoice_map')->insert($dataToInsert);

            DB::table('pre_temp_table')->insert([
                "temp_pid" => $lastPid
            ]);

            return response()->json([
                "status" => "success",
                "data_to_insert" => $dataToInsert
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }

    public function addPreInvoices(Request $request)
    {
        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        /*

        $sql = 'SELECT pim.`temp_pid` FROM pre_temp_table AS pim  ORDER BY id DESC LIMIT 1';
            $pimQuery = mysql_query($sql);
            $pimData = mysql_fetch_assoc($pimQuery);
        $pid =$pid+1;
                mysql_query('insert into pre_invoice_map set pid = \''.$pid.'\' , create_userid = \''.$epcmsLite->userid.'\' , '.($isAgent? 'agent_userid' : 'storeid').' = \''.$storei['s_id'].'\'', $dalilee) or die(mysql_error($dalilee));
        mysql_query('insert into pre_temp_table set temp_pid = \''.$pid.'\'', $dalilee) or die(mysql_error($dalilee));

        */

        try {

            $pre_invoices = $request->pre_invoices;

            foreach ($pre_invoices as $pre_invoice) {

                //  $sql = 'SELECT pim.`temp_pid` FROM pre_temp_table AS pim  ORDER BY id DESC LIMIT 1';

                // $sql = 'SELECT pim.`temp_pid` FROM pre_temp_table AS pim  ORDER BY id DESC LIMIT 1';

                $temp_pid = DB::table('pre_temp_table')->orderBy('id', 'desc')->first();

                $pid = $temp_pid->temp_pid + 1;


                DB::table('pre_invoice_map')->insert([
                    "pid" => $pre_invoice["pid"],
                    "store_id" => $pre_invoice["store_id"],
                ]);
            }
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }



    // api to upload orders from dalilee to current (queue, pickup)

    public function getOrdersPickup(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {
            $requests = DB::table('requests')->whereDate("created_at", ">", date("Y-m-d", strtotime("-150 days")))->where("is_sync", 0)->take(500)->get();


            $requests = $requests->map(function ($request) {


                $store = DB::table('login')->where("s_id", $request->store_id)->first();

                $driver = DB::table('login')->where("s_username", $request->driver_id)->first();

                $area = DB::table('regions_updated')->where("re_id", $request->r_region_id)->first();

                if ($store) {
                    $request->store_code = $store->store_code;
                } else {
                    $request->store_code = null;
                }

                if ($driver) {
                    $request->driver_username = $driver->s_username;
                } else {
                    $request->driver_username = null;
                }

                if ($area) {
                    $request->area_name = $area->re_name;
                } else {
                    $request->area_name = null;
                }


                $year = date("Y", strtotime($request->created_at));
                $month = date("m", strtotime($request->created_at));

                $monthIn3Letters = date("M", strtotime($request->created_at));


                $imagename = $request->pickup_image;

                $request->imageYear = $year;
                $request->imageMonth = $monthIn3Letters;


                $request->imageURL = "https://dalileeom.com/cpdNew/app-api/uploads/$year/$month/$imagename";

                return $request;
            });


            return response()->json([
                "status" => "success",
                "requests" => $requests
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function updateOrdersPickup(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $request_ids = $request->request_ids;

            foreach ($request_ids as $request_id) {
                DB::table('requests')->where("r_id", $request_id)->update([
                    "is_sync" => 1
                ]);
            }

            return response()->json([
                "status" => "success",
                "message" => "successfully synced",
                "request_ids" => $request_ids
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function getCollections(Request $request)
    {

        // if ($request->key!=$this->key) {
        //     return response()->json([
        //         "status"=>"error",
        //         "message"=>"Invalid key"
        //     ]);
        // }

        try {


            // db raw query to get current day collections join with login table to get store code and driver username 

            // $collections=DB::select("SELECT collection.*, dlogin.s_username, login.store_code 
            //                         FROM collection 
            //                         LEFT JOIN login ON login.s_username = collection.store_id 
            //                         LEFT JOIN login as dlogin On dlogin.s_id = collection.driver_id 

            //                         WHERE collection.is_sync = 0 AND collection.is_completed =2 AND collection.create_time >= '".date('Y-m-d')."' LIMIT 1000");

            $collections = DB::table('collection')
                ->join("requests", "requests.co_id", "=", "collection.co_id")
                ->leftJoin("login", "login.s_username", "=", "collection.store_id")
                ->leftJoin("login as dlogin", "dlogin.s_id", "=", "collection.driver_id")
                ->select("collection.*", "login.s_username as store_username", "dlogin.s_username")
                ->where("collection.is_sync", 0)
                ->where("collection.is_completed", 2)
                ->where("collection.create_time", ">=", "2022-07-27")
                ->take(1000)
                ->get();

            // 
            // runsheets 

            $runsheets = DB::select("SELECT run_sheet.*, dlogin.s_username 
                                    FROM run_sheet 
                                    LEFT JOIN login as dlogin On dlogin.s_id = run_sheet.agentid 
                                   
                                    WHERE run_sheet.is_sync = 0 AND run_sheet.create_time >= '" . "2022-07-27" . "' LIMIT 1000");



            // collection requests




            // get requests where co_id is in collection ids

            $co_ids = [];

            foreach ($collections as $collection) {
                $co_ids[] = $collection->co_id;
            }

            $requests = DB::table('requests')
                ->whereIn("co_id", $co_ids)
                ->leftJoin("login", "login.s_username", "=", "requests.store_id")
                ->leftJoin("login as dlogin", "dlogin.s_id", "=", "requests.driver_id")
                ->leftJoin("pre_invoice_map", "pre_invoice_map.actual_id", "=", "requests.r_id")
                ->where("requests.is_sync", 0)
                ->select("requests.*", "login.s_username as store_username", "dlogin.s_username", "pre_invoice_map.*")
                ->get();


            // $remainingRequests = DB::table('requests')
            //     ->join("run_sheet", "run_sheet.rsid", "=", "requests.rsid")
            //     ->leftJoin("login", "login.s_username", "=", "requests.store_id")
            //     ->leftJoin("login as dlogin", "dlogin.s_id", "=", "requests.driver_id")
            //     ->leftJoin("pre_invoice_map", "pre_invoice_map.actual_id", "=", "requests.r_id")
            //     ->where("requests.is_sync", 0)
            //     ->where("run_sheet.create_time", ">=", "2022-07-27")
            //     ->select("requests.*", "login.store_code", "dlogin.s_username", "pre_invoice_map.*")
            //     ->get()
            //     ->take(1000);


            // $allRequests = $requests->merge($remainingRequests);


            $totalRequests = count($requests);


            // foreach($requests as $request){
            //     $r_ids[]=$request->r_id;
            // }

            // $pre_invoices=DB::table('pre_invoice_map')
            //             ->whereIn("actual_id",$r_ids)
            //             ->leftJoin("login", "login.s_id", "=", "pre_invoice_map.storeid")
            //             ->leftJoin("login as dlogin", "dlogin.s_id", "=", "pre_invoice_map.agent_userid")
            //             ->select("pre_invoice_map.*", "login.s_username as store_username", "dlogin.s_username as driver_username")
            //             ->get();



            return response()->json([
                "status" => "success",
                "collections" => $collections,
                "requests" => $requests,
                "totalRequests" => $totalRequests,
                // "pre_invoices"=>$pre_invoices,
                "runsheets" => $runsheets
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function getPreInvoies(Request $request)
    {

        // if ($request->key!=$this->key) {
        //     return response()->json([
        //         "status"=>"error",
        //         "message"=>"Invalid key"
        //     ]);
        // }

        try {
            $pre_invoices = DB::table('pre_invoice_map')->whereDate("create_time", ">", date("Y-m-d", strtotime("-150 days")))->where("is_sync", 0)->take(500)->get();


            $pre_invoices = $pre_invoices->map(function ($pre_invoice) {


                $store = DB::table('login')->where("s_id", $pre_invoice->storeid)->first();

                // $driver=DB::table('login')->where("s_username", $collection->driverid)->first();

                // $area=DB::table('regions_updated')->where("re_id", $collection->region_id)->first();




                if ($store) {
                    $pre_invoice->store_code = $store->store_code;
                } else {
                    $pre_invoice->store_code = null;
                }





                return $pre_invoice;
            });


            return response()->json([
                "status" => "success",
                "pre_invoices" => $pre_invoices
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function updateIsSynced(Request $request)
    {
        // return response()->json($request->all());

        // if ($request->key!=$this->key) {
        //     return response()->json([
        //         "status"=>"error",
        //         "message"=>"Invalid key"
        //     ]);
        // }

        try {

            $ids = $request->ids;

            $table = $request->table;
            $primary_key = $request->primary_key;

            $column = $request->column;




            DB::table($table)->whereIn("rsid", $ids)->update([
                "is_sync" => 1
            ]);



            return response()->json([
                "status" => "success",
                "message" => "successfully synced",
                // "ids" => $ids,
                "table" => $table,
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "table" => $table,
            ]);
        }
    }

    public function updateIsSyncedRequest(Request $request)
    {
        // return response()->json($request->all());

        // if ($request->key!=$this->key) {
        //     return response()->json([
        //         "status"=>"error",
        //         "message"=>"Invalid key"
        //     ]);
        // }

        try {

            $ids = $request->ids;

            $table = $request->table;
            $primary_key = $request->primary_key;

            $column = $request->column;




            DB::table("requests")->whereIn("r_id", $ids)->update([
                "is_sync" => 1
            ]);



            return response()->json([
                "status" => "success",
                "message" => "successfully req synced",
                // "ids" => $ids,
                "table" => $table,
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => "req " . $e->getMessage(),
                "table" => $table,
            ]);
        }
    }


    public function updateIsSyncedCollection(Request $request)
    {
        // return response()->json($request->all());

        // if ($request->key!=$this->key) {
        //     return response()->json([
        //         "status"=>"error",
        //         "message"=>"Invalid key"
        //     ]);
        // }

        try {

            $ids = $request->ids;

            $table = $request->table;
            $primary_key = $request->primary_key;

            $column = $request->column;


            // $collections=DB::table("collection")->whereIn("co_id", $ids)->get();

            $synced = [];

            if (isset($ids)) {

                foreach ($ids as $id) {
                    $totalRequests = DB::table("requests")->where("co_id", $id)->count();

                    $totalIsSyncedRequests = DB::table("requests")->where("co_id", $id)->where("is_sync", 1)->count();

                    if ($totalRequests == $totalIsSyncedRequests) {

                        DB::table("collection")->where("co_id", $id)->update([
                            "is_sync" => 1
                        ]);

                        $synced[] = $id;
                    } else {
                        DB::table("collection")->where("co_id", $id)->update([
                            "is_sync" => 0
                        ]);
                    }
                }
            }


            return response()->json([
                "status" => "success",
                "message" => "successfully coll synced",
                "synced" => $synced,
                "table" => $table,
            ]);
        } catch (\Exception $e) {

            Log::error("exception in update is sync collection method" . $e->getMessage());

            return response()->json([
                "status" => "error",
                "message" => "req " . $e->getMessage(),
                "table" => $table,
            ]);
        }
    }


    public function updateIsDataEntry(Request $request)
    {

        // if ($request->key!=$this->key) {
        //     return response()->json([
        //         "status"=>"error",
        //         "message"=>"Invalid key"
        //     ]);
        // }

        try {

            $ids = $request->ids;

            $table = $request->table;
            $primary_key = $request->primary_key;

            $column = $request->column;


            // 

            DB::table($table)->whereIn("r_id", $ids)->update([
                "shaheen_data_entry" => 1
            ]);



            return response()->json([
                "status" => "success",
                "message" => "successfully data entred",
                "ids" => $ids,
                "column" => $column,
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "column" => $column,
            ]);
        }
    }



    public function updateOrder(Request $request)
    {


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            DB::table("requests")->where("r_id", $request->id)->update([
                "r_name" => $request->name,
                "r_phone" => $request->phone,
                "r_price" => $request->price,
                "cc" => $request->cc,
                "r_ship_price" => $request->ship_price,
                "wilaya_id" => $request->wilaya_id,
                "region_id" => $request->area_id,
                "r_wallayah" => $request->wilaya_id,
                "r_region_id" => $request->area_id,
                "r_region" => $request->area_id,
                "notes" => $request->notes,

            ]);

            if ($request->store_username) {
                DB::table("requests")->where("r_id", $request->id)->update([
                    "store_id" => $request->store_username,
                ]);
            }



            return response()->json([
                "status" => "success",
                "message" => "Order {$request->id} successfully updated",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }

    public function updateOrderCOD(Request $request)
    {


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            DB::table("requests")->where("r_id", $request->id)->update([

                "r_price" => $request->price,

            ]);


            $this->logChange(76, 1, $request->id, [
                'user' => 1069,
                'codam' =>  $request->price
            ], 'operation', 1069, 1);


            return response()->json([
                "status" => "success",
                "message" => "Order {$request->id} successfully updated cod",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function dataEntryOrders(Request $request)
    {



        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $requests =  DB::table("requests")->whereDate("created_at", ">=", "2022-07-19")->where("is_data_entry", 1)->where("shaheen_data_entry", 0)->orderBy("r_id", "desc")->take(1000)->get();



            return response()->json([
                "status" => "success",
                "requests" => $requests,
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function getRemaingingOrdersInRunsheets(Request $request)
    {


        return false;

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $orders = is_array($request->orders) ? $request->orders : null;

            $requests =  DB::table("requests as rq")
                ->leftJoin("pre_invoice_map as pim", "pim.actual_id", "=", "rq.r_id")
                ->leftJoin("login as lg", "lg.s_username", "=", "rq.store_id")
                ->whereIn("rq.rsid", $request->runsheets)
                // ->when($orders, function ($query) use ($orders) {
                //     $query->whereNotIn("rq.r_id", $orders);
                // })
                ->select("rq.*", "pim.*", "lg.store_code as store_code")
                ->get();


            return response()->json([
                "status" => "success",
                "requests" => $requests,
                "res" => $request->all(),
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }



    public function getOrdersForReport(Request $request)
    {



        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $from_date_time = $request->from_date_time;
            $to_date_time = $request->to_date_time;

            $from_sat_date_time = $request->from_sat_date_time;

            $requestsDaily =  DB::table("requests as rq")
                ->whereBetween("rq.created_at", [$from_date_time, $to_date_time])
                ->orderBy("rq.created_at", "asc")
                ->select(DB::raw('CONCAT("A",rq.r_id) as r_id'))
                ->get(["a_id"]);

            $requestsFromSat =  DB::table("requests as rq")
                ->whereBetween("rq.created_at", [$from_sat_date_time, $to_date_time])
                ->orderBy("rq.created_at", "asc")
                ->select(DB::raw('CONCAT("A",rq.r_id) as r_id'))
                ->get(["a_id"]);


            return response()->json([
                "status" => "success",
                "requestsDaily" => $requestsDaily,
                "requestsFromSat" => $requestsFromSat,
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function getOrderData(Request $request)
    {

        return false;


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $r_id = $request->r_id;

            $order_id = strtolower($r_id);

            if (strpos($order_id, "p") !== false) {

                $order_id = trim($order_id, "p");

                $preInvoice = DB::table("pre_invoice_map")->where("pid", $order_id)->first();

                if ($preInvoice) {
                    $r_id = $preInvoice->actual_id;

                    $order =  DB::table("requests as rq")
                        ->leftJoin("pre_invoice_map as pim", "pim.actual_id", "=", "rq.r_id")
                        ->where("rq.r_id", $r_id)
                        ->select("rq.*", "pim.*")
                        ->first();
                }
            } else if (strpos($order_id, "a") !== false) {
                $order_id = trim($order_id, "a");

                $order =  DB::table("requests as rq")
                    ->leftJoin("pre_invoice_map as pim", "pim.actual_id", "=", "rq.r_id")
                    ->where("rq.r_id", $order_id)
                    ->select("rq.*", "pim.*")
                    ->first();
            } else {


                $order =  DB::table("requests as rq")
                    ->leftJoin("pre_invoice_map as pim", "pim.actual_id", "=", "rq.r_id")
                    ->where("rq.r_id", $order_id)
                    ->select("rq.*", "pim.*")
                    ->first();

                if (!$order) {
                    $order =  DB::table("requests as rq")
                        ->leftJoin("pre_invoice_map as pim", "pim.actual_id", "=", "rq.r_id")
                        ->where("rq.external_way_bill_number", $order_id)
                        ->select("rq.*", "pim.*")
                        ->first();
                }
            }




            return response()->json([
                "status" => "success",
                "order" => $order,
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ]);
        }
    }


    public function getExternalOrders(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            // $requests =  DB::table("requests as rq")
            //     ->leftJoin("pre_invoice_map as pim", "pim.actual_id", "=", "rq.r_id")
            //     ->whereNotNull("rq.external_way_bill_number")
            //     ->where("rq.external_way_bill_number", "!=", "")
            //     ->where("rq.created_at", ">=", "2022-07-27")
            //     ->select("rq.*", "pim.*")
            //     ->get()
            //     ;


            return response()->json([
                "status" => "stop",
                // "requests" => $requests,
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function getStore(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $newStore = "no";

            $store = DB::table("login")
                ->where("s_type", "store")
                ->where("s_mobile", $request->mobile)
                ->first();

            if (!$store) {

                $lastStoreCode = DB::table("login")
                    ->where("s_type", "store")
                    ->orderBy("s_id", "desc")
                    ->first();

                if ($lastStoreCode) {
                    $lastStoreCode = $lastStoreCode->store_code;
                } else {
                    $lastStoreCode = 0;
                }
                $incStoreCode = intval(substr($lastStoreCode, 2)) + 1;

                $storeCodeNumber = str_pad($incStoreCode, 5, '0', STR_PAD_LEFT);

                $storeCode = trim("ST" . $storeCodeNumber);

                $store_id = DB::table("login")
                    ->insertGetId(
                        [
                            "s_username" => $request->mobile,
                            "s_mobile" => $request->mobile,
                            "store_code" => $storeCode,
                            "s_name" => $request->mobile,
                            "s_type" => "store",
                        ]
                    );

                $store = DB::table("login")
                    ->where("s_id", $store_id)
                    ->first();


                $newStore = "yes";
            }


            return response()->json([
                "status" => "success",
                "store" => $store,
                "new_store" => $newStore,
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }

    public function getStoreByEmail(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $newStore = "no";

            $store = DB::table("login")
                ->where("s_type", "store")
                ->where("s_email", $request->email)
                ->first();

            if (!$store) {

                $lastStoreCode = DB::table("login")
                    ->where("s_type", "store")
                    ->orderBy("s_id", "desc")
                    ->first();

                if ($lastStoreCode) {
                    $lastStoreCode = $lastStoreCode->store_code;
                } else {
                    $lastStoreCode = 0;
                }
                $incStoreCode = intval(substr($lastStoreCode, 2)) + 1;

                $storeCodeNumber = str_pad($incStoreCode, 5, '0', STR_PAD_LEFT);

                $storeCode = trim("ST" . $storeCodeNumber);

                $store_id = DB::table("login")
                    ->insertGetId(
                        [
                            "s_username" => $request->email,
                            "s_email" => $request->email,
                            "store_code" => $storeCode,
                            "s_name" => $request->email,
                            "s_type" => "store",
                        ]
                    );

                $store = DB::table("login")
                    ->where("s_id", $store_id)
                    ->first();


                $newStore = "yes";
            }


            return response()->json([
                "status" => "success",
                "store" => $store,
                "new_store" => $newStore,
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }




    public function addOrdersOfflineDriverPickup(Request $request)
    {

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ]);
        }

        try {

            $collections = $request->collections;

            $driver_username = $request->driver_username;

            $data = [];

            $user_id = $request->user_id;

            foreach ($collections as $collection) {


                $refrerence = DB::table("collection")->where('co_id', $collection["collection_id"])->first();

                if ($refrerence && $refrerence->is_paid == 0) {

                    $refrerence->update([
                        "collected_cash" => $collection["cop"],
                        "is_completed" => 2,
                    ]);

                    $order_ids = $collection['order_id'] ? $collection['order_id'] : [];

                    $pickup_images = $collection['pickup_image'] ? $collection['pickup_image'] : [];




                    foreach ($order_ids as $index => $order_id) {
                        $pickup_image = $pickup_images[$index];

                        $order_id = strtolower($order_id);

                        if (strpos($order_id, "p") !== false) {
                            $pid = trim($order_id, "p");

                            $pre_invoice =  DB::table("pre_invoice_map")->where("pid", $pid)->first();

                            if ($pre_invoice) {

                                $requestR =  DB::table("requests")->where("r_id", $pre_invoice->actual_id)->first();


                                if (!$requestR) {
                                }
                            }
                        }
                    }
                }
            }


            return response()->json([
                "status" => "success",
                // "data" => $data,
                "collections" => $collections,
                "message" => "successfully ",
            ]);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage()
            ]);
        }
    }


    public function getStoresOrdersCount(Request $request)
    {

        $request->validate([
            "key" => "required",
            "store_username" => "required|exists:login,s_username"
        ]);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }

        try {

            $store_username = $request->store_username;

            $stats = DB::select("SELECT 
                                count(r_id) as total_shipments, 
                                SUM(case when r_status='completed' then 1 else 0 END) as deliverd_shipments,
                                SUM(case when r_status='A' and driver_id!=NULL then 1 else 0 END) as to_be_delivered,
                                SUM(case when r_status='return' then 1 else 0 END) as returned_shipments,
                                SUM(case when r_status='canceled' then 1 else 0 END) as cancel_shipments,
                                SUM(case when r_status!='completed' and r_status!='canceled' and r_status!='return' and r_status!='OFD' then 1 else 0 END) as undelivered_shipments,
                                SUM(case when r_status='OFD' then 1 else 0 END) as ofd_shipments
                                
            
                        from requests where is_hide=0 and record_status=1 and store_id='$store_username' ");

            $tracking_statuses = DB::table("order_tracking_statuses")->get();

            $stats[0]->total_shipments = (int)$stats[0]->total_shipments;
            $stats[0]->deliverd_shipments = (int)$stats[0]->deliverd_shipments;
            $stats[0]->to_be_delivered = (int)$stats[0]->to_be_delivered;
            $stats[0]->returned_shipments = (int)$stats[0]->returned_shipments;
            $stats[0]->undelivered_shipments = (int)$stats[0]->undelivered_shipments;
            $stats[0]->cancel_shipments = (int)$stats[0]->cancel_shipments;
            $stats[0]->ofd_shipments = (int)$stats[0]->ofd_shipments;

            $tracking_statuses = $tracking_statuses->map(function ($status) {

                return [
                    "id" => $status->id,
                    "status_ar" => $status->status_ar,
                    "status_eng" => $status->status_eng,
                    "icon" => "https://shaheen-oman.dalilee.om/storage/order-icons/" . $status->icon,

                ];
            });



            return response()->json([
                "status" => "success",
                "data" => [
                    "stats" => $stats[0],
                    "tracking_statuses" => $tracking_statuses
                ],
                "message" => "successfully ",
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ], 422);
        }
    }

    public function getStoresOrders(Request $request)
    {

        $request->validate([
            "key" => "required",
            "module" => "required|in:shipments,delivered_shipments,undelivered_shipments,return_shipments,cancel_shipments,ofd_shipments,open_finance,shipper_self_orders",
            "offset" => "nullable|integer",
            "limit" => "nullable|integer",
            "store_username" => "required|exists:login,s_username",
            "collection_id" => "nullable|exists:collection,co_id"
        ]);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }

        try {

            $store_username = $request->store_username;
            $module = $request->module;

            $from_date = $request->from_date;
            $to_date = $request->to_date;

            $total_requests = 0;
            $requests = null;

            $offset = $request->offset;
            $limit = $request->limit;

            $collection_id = null;


            switch ($module) {

                case 'shipments':

                    $status = false;
                    break;

                case 'delivered_shipments':

                    $status = "completed";
                    break;

                case 'undelivered_shipments':

                    $status = "F";
                    break;
                case 'return_shipments':

                    $status = "return";
                    break;
                case 'cancel_shipments':

                    $status = "canceled";
                    break;

                case 'ofd_shipments':

                    $status = "OFD";
                    break;

                case 'open_finance':

                    $status = "open_finance";
                    break;

                case 'shipper_self_orders':

                    $status="shipper_self_orders";
                    break;

                default:
                    $status = false;
                    break;
            }

            
            
            

            $total_requests = 0;
            if (!$from_date) {
                $total_requests = DB::table("requests")
                    ->where("is_hide", 0)
                    ->where("record_status", 1)
                    ->where("store_id", $store_username)

                    ->when($status == "open_finance", function ($q) {
                        $q->where("is_paid", 1)
                            ->whereNull("r_closing_date")
                            ->where("r_status", "completed");
                    })

                    ->when($status != false && $status != "F" && $status != "shipper_self_orders", function ($q) use ($status) {
                        $q->where("r_status", $status);
                    })
                    ->when($status == "completed", function ($q) use ($status) {
                        // $q->where("is_paid", 1);
                    })
                    ->when($status == "OFD", function ($q) use ($status) {
                        $q->where("r_status", "=", "OFD");
                    })
                    ->when($status == "F", function ($q) use ($status) {
                        $q->where("r_status", "!=", "completed")
                            ->where("r_status", "!=", "canceled")
                            ->Where("r_status", "!=", "return")
                            ->where("r_status", "!=", "OFD");
                    })
                    ->when($status == "shipper_self_orders", function ($q) use ($status,$store_username) {
                   
                        $q->where("added_by_store",1);
                    })
                    ->count();
            }


            $requests = DB::table("requests as rq")
                ->leftJoin("collection as col", "rq.co_id", "col.co_id")
                // ->leftJoin("login as dl", "rq.driver_id", "dl.s_username")
                ->leftJoin("pre_invoice_map as pim", "rq.r_id", "pim.actual_id")
                ->leftJoin("order_problems as op", "rq.r_id", "op.order_id")
                ->leftJoin("problem_reasons as pr", "op.problem_reason_id", "pr.id")
                ->leftJoin("wilaya as w", "rq.r_wallayah", "w.id")
                ->leftJoin("regions as rg", "rq.r_region", "rg.re_id")
                ->where("rq.is_hide", 0)
                ->where("rq.record_status", 1)
                ->where("rq.store_id", $store_username)
                ->when($from_date, function ($q) use ($from_date, $to_date) {
                  
                    $q->whereBetween("rq.created_at", [$from_date, $to_date]);
                })

                ->when($status == "open_finance", function ($q) {
                  
                    $q->where("rq.is_paid", 1)
                        ->whereNull("rq.r_closing_date")
                        ->where("rq.r_status", "completed");
                })

                
                ->when($status == "completed", function ($q) use ($status) {
                   
                    return  $q->where("rq.is_paid", 1);
                })
                ->when($status == "F", function ($q) use ($status) {
                   
                    $q->where("rq.r_status", "!=", "completed")
                        ->where("rq.r_status", "!=", "canceled")
                        ->Where("rq.r_status", "!=", "return");
                })
              
                ->when($status == "shipper_self_orders", function ($q) use ($status) {
                   
                    $q->where("rq.added_by_store", 1);
                })
                ->when($status == "OFD", function ($q) use ($status) {
                   
                    $q->where("rq.r_status", "=", "OFD");
                })
                ->when($status != false && $status != "F"  && $status != "shipper_self_orders", function ($q) use ($status) {
                   
                    return $q->where("rq.r_status", $status);
                })
                ->select(
                  
                    "rq.r_id as order_id",
                    DB::raw('CONCAT("P",pim.pid) as order_no'),
                    "rq.co_id as ref_id",
                    "col.collected_cash as cop",
                    "rq.r_phone as customer_no",
                    "rq.r_name as customer_name",
                    "rq.cc as cc",
                    "rq.r_status as order_status_name",
                    "rq.r_status as order_status_key",
                    "rq.pickup_image as order_pickup_image",
                    "rq.order_image as order_deliver_image",
                    "rq.undeliver_image as order_undeliver_image",
                    "rq.undeliver_image2 as order_undeliver_image2",
                    "rq.undeliver_image3 as order_undeliver_image3",
                    "rq.weight as weight",
                    "rq.r_phone as phone",
                    "rq.r_ship_price as shipping_price",
                    "rq.r_price as cod",
                    "w.name_en as wilaya_name",
                    "rg.re_name as area_name",
                    "rq.added_by_store",
                    "rq.store_id",
                    "op.problem_reason_id",
                    "pr.title as problem_reasons",
                    "op.trader_comments as trader_comments",
                    "op.id as problem_id",  
                     'op.auditor_response as problem_response',                 
                    // "dl.s_name as driver_name",
                    "rq.external_way_bill_number",
                    DB::raw("DATE_FORMAT(rq.created_at,'%Y-%m-%d') as created_at"),
                    DB::raw("DATE_FORMAT(rq.updated_at,'%Y-%m-%d') as updated_at")

                )
                ->when(!$from_date, function ($q) use ($offset, $limit) {
                    $q->skip($offset)
                        ->take($limit);
                })
                ->orderBy("rq.updated_at", "desc")
                ->get();

               
            $resp = $this->processOrders($total_requests, $requests);

            return response()->json([
                "status" => "success",
                "data" => [
                    "total_shipments" => $resp["total_requests"],
                    "shipments" => $resp["data"],
                ]
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ], 422);
        }
    }


    public function processOrders($total_requests, $requests)
    {
        try {

            $logs = DB::table("change_log as log")
                ->leftJoin("branch as br", "log.branchid", "br.branchid")
                ->whereIn("log.parentid", $requests->pluck("order_id"))
                ->select("log.*", "br.name_en as branch_name")
                ->orderBy("log.create_time", "desc")
                ->get();


            $grouped_logs = [];

            $activties = [];


            $current_status = 5;
            foreach ($logs as $log) {

                $grouped_logs[$log->parentid][] = $log;

                $module = $log->module;
                $branch_name = $log->branch_name;
                $decription = "";

                if ($module == "send_sheet") {
                    $decription = "Order sent to " . $branch_name;
                    $current_status = 2;
                }

                // if ($module == "logSheet") {
                //     $decription = "Order data entry done ";
                // }

                if ($module == "receive_sheet") {
                    $decription = "Order received at " . $branch_name;
                    $current_status = 3;
                }

                if ($module == "runsheet") {
                    $decription = "Order assigned to driver, out for delivery ";
                    $current_status = 4;
                }

                if ($module == "undelivered") {
                    $decription = "Order is undelivered ";
                    $current_status = 6;
                }

                if ($module == "delivered") {
                    $decription = "Order is delivered ";
                    $current_status = 7;
                }


                if ($decription != "") {

                    $activties[$log->parentid][] = [
                        "id" => $log->clid,
                        "status" => $current_status,
                        "external_text" => $decription,
                        "created_at" => $log->create_time,
                        "updated_at" =>  $log->create_time,
                    ];
                }
            }



            $data = [];
            $tempReq = [];
            foreach ($requests as $request) {

                // if (!in_array($request->order_id, $tempReq)) {


                if ($request->order_no == "" || is_null($request->order_no)) {
                    $request->order_no = $request->external_way_bill_number ? $request->external_way_bill_number : "A" . $request->order_id;
                }

                $order = $request;
                // $order_activities = $request->logs;
                // $activties = [];
                // $current_status = 1;

                // $request->logs = $grouped_logs[$request->order_id];
                $order->order_activities = @$activties[$request->order_id];

                $statuses = [];
                if ($order->order_activities) {
                    $statuses = (array_unique(array_column($order->order_activities, "status")));
                }
                $order->status = $statuses;
                if (empty($statuses)) {
                    $order->current_status = $this->retrieveStatus($order->order_status_key)["status_name"]["id"];
                } else {
                    $order->current_status = count($statuses) > 0 ? max($statuses) : 5;
                }


                $order->order_pickup_image = $this->getOrderImagesDalilee(["image_name" => $order->order_pickup_image, "created_at" => $order->created_at]);

                $order->order_deliver_image = $this->getOrderImagesDalilee(["image_name" => $order->order_deliver_image, "created_at" => $order->created_at]);

                $order->order_undeliver_image = $this->getOrderImagesDalilee(["image_name" => $order->order_undeliver_image, "created_at" => $order->created_at]);

                $order->order_undeliver_image2 = $this->getOrderImagesDalilee(["image_name" => $order->order_undeliver_image2, "created_at" => $order->created_at]);

                $order->order_undeliver_image3 = $this->getOrderImagesDalilee(["image_name" => $order->order_undeliver_image3, "created_at" => $order->created_at]);

                //
                $order_status_name = $this->retrieveStatus($order->order_status_key)["status_name"]["value"];

                $order->order_status_name = $order_status_name;

                $data[] = $order;

                $tempReq[] = $order->order_id;
                // }
            }



            return [
                "total_requests" => $total_requests,
                "data" => $data
            ];
        } catch (\Exception $e) {

            dd($e->getTraceAsString());
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ]);
        }
    }


    public function getOrderImagesDalilee($params)
    {
        $image = $params["image_name"];

        if ($image && $image != "") {
            $created_at = (strtotime($params["created_at"]));

            $month = date("m", $created_at);

            $year = date("Y", $created_at);


            $url = "https://dalilee.net/cpdNew/test-api/uploads/$year/$month/$image";

            return $url;
        } else {
            return "";
        }
    }



    public function getAmount($store_username, $is_booked = null, $invoice_id = null)
    {

        $completedRequests =  DB::table("requests")
            ->join("run_sheet as rs", "requests.rsid", "rs.rsid")
            ->where("store_id", $store_username)
            ->where("r_status", "completed")
            ->when($is_booked, function ($q) use ($is_booked) {
                $q->where("r_booked", $is_booked);
            })->when($invoice_id, function ($q) use ($invoice_id) {
                $q->where("invoice_id", ">", 0);
            })
            ->where("rs.is_paid", 1)
            ->get();


        $returnRequests =  DB::table("requests")
            ->where("store_id", $store_username)
            ->where("r_status", "return")
            ->where("r_booked", $is_booked)
            ->get();


        $cod = 0;
        $cc = 0;

        $ship_price = 0;

        foreach ($completedRequests as $request) {
            $cod += $request->r_price;
            $cc += $request->cc;
            $ship_price += $request->r_ship_price;
        }

        foreach ($returnRequests as $request) {
            $cc += $request->cc;
            $ship_price += $request->r_ship_price;
        }

        // dd($cod, $cc,$ship_price,$completedRequests->count(),$returnRequests->count(),$is_booked);

        return ($cod + $cc) - ($ship_price);
    }


    public function storeFinanceDashboard(Request $request)
    {

        $request->validate([
            "key" => "required",
            "store_username" => "required|exists:login,s_username"
        ]);

        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }


        $store_username = $request->store_username;

        $store = DB::table('login')->where("s_username", $store_username)->first();





        $rowCollectionData = DB::table("requests as rq")

            ->leftJoin("collection as col", "rq.co_id", "col.co_id")
            ->leftJoin("run_sheet as rs", "rq.rsid", "rs.rsid")
            ->join("login as lg", "rq.store_id", "lg.s_username")

            ->where("rq.store_id", $store_username)
            ->where("lg.s_id", $store->s_id)
            ->where("rq.is_hide", 0)
            ->where("rq.record_status", 1)
            ->whereDate("rq.created_at", ">", "2022-01-07")
            ->select(

                // DB::raw("SUM(rq.r_price) as total_orders_amount"),

                DB::raw("SUM(CASE when rq.r_status='completed' and rq.force_closing=0 and rs.is_paid=0 and (rq.r_booked='no')  then rq.r_price  else 0 END) as codWithDriverTotalCOD"),
                // DB::raw("SUM(CASE when rq.r_status='completed' and rq.force_closing=0 and rs.is_paid=0 and (rq.r_booked='no')  then rq.r_ship_price  else 0 END) as codWithDriverTotalShipping"),

                // DB::raw("SUM(CASE when rq.r_status='completed' and rq.force_closing=0 and rs.is_paid=0 and (rq.r_booked='no')  then rq.cc  else 0 END) as codWithDriverTotalCC"),

                DB::raw("SUM(rq.r_price ) as ordersCOD"),
                DB::raw("SUM(rq.r_ship_price ) as ordersShipPrice"),
                DB::raw("SUM(rq.cc ) as ordersCC"),

                DB::raw("SUM(CASE when rq.r_booked='yes' or rq.invoice_id>0 then rq.r_price  else 0 END) as ordersBookedCOD"),
                DB::raw("SUM(CASE when rq.r_booked='yes' or rq.invoice_id>0 then rq.r_ship_price  else 0 END) as ordersBookedShipPrice"),
                DB::raw("SUM(CASE when rq.r_booked='yes' or rq.invoice_id>0 then rq.cc  else 0 END) as ordersBookedCC"),


                DB::raw("SUM(CASE when rq.invoice_id=0 and col.co_id>1101 and rq.r_status='completed' and (rq.force_closing=1 OR rs.is_paid=1) and (rq.r_booked='no' OR rq.r_booked is null)  then rq.cc  else 0 END) as totalccCompleted"),
                DB::raw("SUM(CASE when rq.invoice_id=0 and col.co_id>1101 and rq.r_status='return' and (rq.r_booked='no' OR rq.r_booked is null)  then rq.cc  else 0 END) as totalccReturn"),

                DB::raw("SUM(CASE when rq.invoice_id=0 and col.co_id>1101 and rq.r_status='completed' and (rq.force_closing=1 OR rs.is_paid=1) and (rq.r_booked='no' OR rq.r_booked is null)  then rq.r_ship_price  else 0 END) as totalShipingPriceCompleted"),
                DB::raw("SUM(CASE when rq.invoice_id=0 and col.co_id>1101 and rq.r_status='return' and (rq.r_booked='no' OR rq.r_booked is null)  then rq.r_ship_price  else 0 END) as totalShipingPriceReturn"),

                DB::raw("SUM(CASE when rq.invoice_id=0 and col.co_id>1101 and rq.r_status='completed' and (rq.force_closing=1 OR rs.is_paid=1) and (rq.r_booked='no' OR rq.r_booked is null)  then rq.r_price  else 0 END) as totalCodCompleted"),

                DB::raw("SUM(case when rq.r_status='return' then rq.r_price else 0 END) as cod_returned"),

                DB::raw("SUM(case when rq.r_status!='OFD' and rq.r_status!='return' and rq.r_status!='cancel' and rq.r_status!='completed' then rq.r_price else 0 END) as cod_pending")

                // DB::raw("SUM(case when rq.r_status='completed' and rq.store_approved='yes' and lg.company_id=1  and rq.driver_id!='' then rq.r_price else 0 END) as old_invoice_total_price"),
                // DB::raw("SUM(case when rq.r_status='completed' and rq.store_approved='yes' and lg.company_id=1  and rq.driver_id!='' then rq.cc else 0 END) as old_invoice_cc"),
                // DB::raw("SUM(case when rq.r_status='completed' and rq.store_approved='yes' and lg.company_id=1  and rq.driver_id!='' then rq.r_ship_price else 0 END) as old_invoice_shipping")

            )
            ->first();



        $cod_with_driver = $rowCollectionData->codWithDriverTotalCOD;

        $cod = $rowCollectionData->ordersCOD;
        $shippingPrice = $rowCollectionData->ordersShipPrice;
        $cc = $rowCollectionData->ordersCC;

        $codBooked = $rowCollectionData->ordersBookedCOD;
        $shippingPriceBooked = $rowCollectionData->ordersBookedShipPrice;
        $ccBooked = $rowCollectionData->ordersBookedCC;


        $total_orders_amount = $cod - $shippingPrice - $cc;

        $paid = $codBooked - $shippingPriceBooked - $ccBooked;

        $total_shipping_amount = $shippingPriceBooked;


        $cod_returned = $rowCollectionData->cod_returned;
        $cod_pending = $rowCollectionData->cod_pending;



        // ready to pay starts

        $cc = $rowCollectionData->totalccCompleted + $rowCollectionData->totalccReturn;
        $shipping = $rowCollectionData->totalShipingPriceCompleted;
        $balance = $cc + $rowCollectionData->totalCodCompleted - $shipping;

        $ready_to_pay = $balance;

        // ready to pay ends



        $stats = [
            "total_amount" => round($total_orders_amount, 2),
            "paid" => round($paid, 2),
            "cod_pending" => round($cod_pending, 2),
            "ready_to_pay" => round($ready_to_pay, 2),
            "cod_with_drivers" => round($cod_with_driver, 2),
            "cod_returned" => round($cod_returned, 2),
            "total_shipping_amount" => floatval("-" . round($total_shipping_amount, 2)),
        ];



        return response()->json([
            "success" => "ok",
            "data" => [
                "stats" => $stats
            ]

        ], 200);
    }


    public function storeFinanceOrders(Request $request)
    {

        $request->validate([
            "key" => "required",
            "module" => "required|in:total_amount,paid,ready_to_pay,cod_with_drivers,cod_pending,cod_returned",
            "offset" => "required|integer",
            "limit" => "required|integer",
            "store_username" => "required|exists:login,s_username"
        ]);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }

        try {

            $store_username = $request->store_username;
            $module = $request->module;


            $total_requests = 0;
            $requests = null;

            $offset = $request->offset;
            $limit = $request->limit;


            switch ($module) {

                case 'total_amount':
                    $requests = AppRequest::leftJoin("collection as col", "requests.co_id", "col.co_id")
                        ->leftJoin("pre_invoice_map as pim", "requests.r_id", "pim.actual_id")
                        ->where("requests.is_hide", 0)
                        ->where("requests.record_status", 1)
                        ->where("requests.store_id", $store_username)
                        ->where("requests.r_booked", "yes")
                        ->where(function ($q) {
                            $q->where("requests.r_status", "completed")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1);
                                });
                        })
                        ->orWhere(function ($q) {
                            $q->where("requests.r_status", "return")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1)
                                        ->orWhere("is_paid", 0);
                                });
                        })->orderBy("requests.updated_at", "desc")
                        ->select(
                            "requests.r_id as order_id",
                            DB::raw('CONCAT("P",pim.pid) as order_no'),
                            "requests.co_id as ref_id",
                            "col.collected_cash as cop",
                            "requests.r_phone as customer_no",
                            "requests.r_name as customer_name",
                            "requests.cc as cc",
                            "requests.r_status as order_status_name",
                            "requests.r_status as order_status_key",
                            "requests.pickup_image as order_pickup_image",
                            "requests.order_image as order_deliver_image",
                            "requests.undeliver_image as order_undeliver_image",
                            "requests.weight as weight",
                            "requests.r_phone as phone",
                            "requests.r_ship_price as shipping_price",
                            "requests.r_price as cod",
                            "requests.external_way_bill_number",
                            DB::raw("DATE_FORMAT(requests.created_at,'%Y-%m-%d') as created_at"),
                            DB::raw("DATE_FORMAT(requests.updated_at,'%Y-%m-%d') as updated_at")
                        )
                        ->skip($offset)->take($limit)->get();



                    $total_requests = AppRequest::where("is_hide", 0)
                        ->where("record_status", 1)
                        ->where("store_id", $store_username)
                        ->where("r_booked", "yes")
                        ->where(function ($q) {
                            $q->where("r_status", "completed")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1);
                                });
                        })
                        ->orWhere(function ($q) {
                            $q->where("r_status", "return")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1)
                                        ->orWhere("is_paid", 0);
                                });
                        })->count();

                    break;

                case 'paid':

                    $requests = AppRequest::leftJoin("collection as col", "requests.co_id", "col.co_id")
                        ->leftJoin("pre_invoice_map as pim", "requests.r_id", "pim.actual_id")
                        ->where("requests.is_hide", 0)
                        ->where("requests.record_status", 1)
                        ->where("requests.store_id", $store_username)
                        ->where(function ($q) {
                            $q->where("requests.r_status", "completed")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1);
                                });
                        })
                        ->orWhere(function ($q) {
                            $q->where("requests.r_status", "return")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1)
                                        ->orWhere("is_paid", 0);
                                });
                        })
                        ->select(
                            "requests.r_id as order_id",
                            DB::raw('CONCAT("P",pim.pid) as order_no'),
                            "requests.co_id as ref_id",
                            "col.collected_cash as cop",
                            "requests.r_phone as customer_no",
                            "requests.r_name as customer_name",
                            "requests.cc as cc",
                            "requests.r_status as order_status_name",
                            "requests.r_status as order_status_key",
                            "requests.pickup_image as order_pickup_image",
                            "requests.order_image as order_deliver_image",
                            "requests.undeliver_image as order_undeliver_image",
                            "requests.weight as weight",
                            "requests.r_phone as phone",
                            "requests.r_ship_price as shipping_price",
                            "requests.r_price as cod",
                            "requests.external_way_bill_number",
                            "requests.created_at as created_at",
                            "requests.updated_at as updated_at"
                        )
                        ->orderBy("updated_at", "desc")
                        ->skip($offset)->take($limit)->get();

                    $total_requests = AppRequest::where("is_hide", 0)
                        ->where("record_status", 1)
                        ->where("store_id", $store_username)
                        ->where(function ($q) {
                            $q->where("r_status", "completed")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1);
                                });
                        })
                        ->orWhere(function ($q) {
                            $q->where("r_status", "return")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1)
                                        ->orWhere("is_paid", 0);
                                });
                        })->count();

                    break;

                case 'ready_to_pay':


                    $requests = AppRequest::leftJoin("collection as col", "requests.co_id", "col.co_id")
                        ->leftJoin("pre_invoice_map as pim", "requests.r_id", "pim.actual_id")
                        ->where("requests.is_hide", 0)
                        ->where("requests.record_status", 1)
                        ->where("requests.store_id", $store_username)
                        ->where("requests.r_booked", "no")
                        ->where(function ($q) {
                            $q->where("r_status", "completed")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1);
                                });
                        })
                        ->orWhere(function ($q) {
                            $q->where("r_status", "return")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1)
                                        ->orWhere("is_paid", 0);
                                });
                        })->select(
                            "requests.r_id as order_id",
                            DB::raw('CONCAT("P",pim.pid) as order_no'),
                            "requests.co_id as ref_id",
                            "col.collected_cash as cop",
                            "requests.r_phone as customer_no",
                            "requests.r_name as customer_name",
                            "requests.cc as cc",
                            "requests.r_status as order_status_name",
                            "requests.r_status as order_status_key",
                            "requests.pickup_image as order_pickup_image",
                            "requests.order_image as order_deliver_image",
                            "requests.undeliver_image as order_undeliver_image",
                            "requests.weight as weight",
                            "requests.r_phone as phone",
                            "requests.r_ship_price as shipping_price",
                            "requests.r_price as cod",
                            "requests.external_way_bill_number",
                            "requests.created_at as created_at",
                            "requests.updated_at as updated_at"
                        )->orderBy("requests.updated_at", "desc")
                        ->skip($offset)->take($limit)->get();


                    $total_requests = AppRequest::where("is_hide", 0)
                        ->where("record_status", 1)
                        ->where("store_id", $store_username)
                        ->where("r_booked", "no")
                        ->where(function ($q) {
                            $q->where("r_status", "completed")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1);
                                });
                        })
                        ->orWhere(function ($q) {
                            $q->where("r_status", "return")
                                ->whereHas("runSheet", function ($qr) {
                                    $qr->where("is_paid", 1)
                                        ->orWhere("is_paid", 0);
                                });
                        })->count();

                    break;
                case 'cod_with_drivers':
                    $requests =  AppRequest::leftJoin("collection as col", "requests.co_id", "col.co_id")
                        ->leftJoin("pre_invoice_map as pim", "requests.r_id", "pim.actual_id")
                        ->where("requests.store_id", $store_username)
                        ->where("requests.r_status", "completed")
                        ->where("requests.is_paid", 0)
                        ->select(
                            "requests.r_id as order_id",
                            DB::raw('CONCAT("P",pim.pid) as order_no'),
                            "requests.co_id as ref_id",
                            "col.collected_cash as cop",
                            "requests.r_phone as customer_no",
                            "requests.r_name as customer_name",
                            "requests.cc as cc",
                            "requests.r_status as order_status_name",
                            "requests.r_status as order_status_key",
                            "requests.pickup_image as order_pickup_image",
                            "requests.order_image as order_deliver_image",
                            "requests.undeliver_image as order_undeliver_image",
                            "requests.weight as weight",
                            "requests.r_phone as phone",
                            "requests.r_ship_price as shipping_price",
                            "requests.r_price as cod",
                            "requests.external_way_bill_number",
                            "requests.created_at as created_at",
                            "requests.updated_at as updated_at"
                        )->orderBy("updated_at", "desc")
                        ->skip($offset)
                        ->take($limit)
                        ->get();
                    $total_requests =  AppRequest::where("store_id", $store_username)->where("r_status", "completed")->where("is_paid", 0)->count();

                    break;
                case 'cod_pending':

                    $requests =  AppRequest::leftJoin("collection as col", "requests.co_id", "col.co_id")
                        ->leftJoin("pre_invoice_map as pim", "requests.r_id", "pim.actual_id")
                        ->where("requests.store_id", $store_username)
                        ->whereNull("requests.rsid")
                        ->select(
                            "requests.r_id as order_id",
                            DB::raw('CONCAT("P",pim.pid) as order_no'),
                            "requests.co_id as ref_id",
                            "col.collected_cash as cop",
                            "requests.r_phone as customer_no",
                            "requests.r_name as customer_name",
                            "requests.cc as cc",
                            "requests.r_status as order_status_name",
                            "requests.r_status as order_status_key",
                            "requests.pickup_image as order_pickup_image",
                            "requests.order_image as order_deliver_image",
                            "requests.undeliver_image as order_undeliver_image",
                            "requests.weight as weight",
                            "requests.r_phone as phone",
                            "requests.r_ship_price as shipping_price",
                            "requests.r_price as cod",
                            "requests.external_way_bill_number",
                            "requests.created_at as created_at",
                            "requests.updated_at as updated_at"
                        )
                        ->orderBy("requests.updated_at", "desc")->skip($offset)->take($limit)->get();
                    $total_requests =  AppRequest::where("store_id", $store_username)->whereNull("rsid")->count();

                    break;

                case 'cod_returned':

                    $requests =  AppRequest::leftJoin("collection as col", "requests.co_id", "col.co_id")
                        ->leftJoin("pre_invoice_map as pim", "requests.r_id", "pim.actual_id")
                        ->where("requests.store_id", $store_username)
                        ->where("requests.r_status", "return")->select(
                            "requests.r_id as order_id",
                            DB::raw('CONCAT("P",pim.pid) as order_no'),
                            "requests.co_id as ref_id",
                            "col.collected_cash as cop",
                            "requests.r_phone as customer_no",
                            "requests.r_name as customer_name",
                            "requests.cc as cc",
                            "requests.r_status as order_status_name",
                            "requests.r_status as order_status_key",
                            "requests.pickup_image as order_pickup_image",
                            "requests.order_image as order_deliver_image",
                            "requests.undeliver_image as order_undeliver_image",
                            "requests.weight as weight",
                            "requests.r_phone as phone",
                            "requests.r_ship_price as shipping_price",
                            "requests.r_price as cod",
                            "requests.external_way_bill_number",
                            "requests.created_at as created_at",
                            "requests.updated_at as updated_at"
                        )->orderBy("requests.updated_at", "desc")
                        ->skip($offset)->take($limit)->get();

                    $total_requests =  AppRequest::where("store_id", $store_username)->where("r_status", "return")->count();


                    break;

                default:
                    # code...
                    $requests = [];
                    $total_requests = 0;
                    break;
            }


            $resp = $this->processOrders($total_requests, $requests);

            return response()->json(["success" => "ok", "data" => [
                "total_shipments" => $resp["total_requests"],
                "shipments" => $resp["data"],
            ]], 200);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ], 422);
        }
    }


    public function retrieveStatus($status)
    {

        $status_name = null;

        $statuses = [
            'A' => ['value' => 'Being processed', 'id' => 5],
            'I' => ['value' => 'Order not collected', 'id' => 5],
            'completed' => ['value' => 'Delivered', 'id' => 7],
            'canceled' =>  ['value' => 'Canceled', 'id' => 5],
            'postpone' =>  ['value' => 'postpone', 'id' => 5],
            'noresponse' =>  ['value' => 'No Response', 'id' => 5],
            'reroute' =>  ['value' => 'Redirection', 'id' => 5],
            'F' =>  ['value' => 'Undelivered', 'id' => 6], // New
            'OFD' =>  ['value' => 'OFD', 'id' => 5],
            'FW' =>  ['value' => 'Undelivered Back to Warehouse', 'id' => 6],
            'BR' =>  ['value' => 'Returned to Branch', 'id' => 5], // New
            'return' =>  ['value' => 'Return to trader', 'id' => 5],
            'FS' =>  ['value' => 'Undelivered Self Pickup', 'id' => 6],
            'Fatmp1' =>  ['value' => 'Undelivered call atemp 1', 'id' => 6],
            'Fatmp2' =>  ['value' => 'Undelivered call atemp 2', 'id' => 6],
            'Utraderreport1' =>  ['value' => 'undeliverd trader report 1 ', 'id' => 6],
            'Utraderreport2' =>  ['value' => 'undeliverd trader report 2 ', 'id' => 6],
            'FD' =>  ['value' => 'Undelivered ORDER DAMAGED', 'id' => 6],
            'refund' =>  ['value' => 'Refund', 'id' => 6],
            'reschedule' =>  ['value' => 'Reschedule', 'id' => 5],
            'schedule' =>  ['value' => 'Schedule for Delivery', 'id' => 5],
            'hold' =>  ['value' => 'hold at office', 'id' => 5],
            'pickupbydriver' =>  ['value' => 'Pickuped', 'id' => 1], // New
            'intransit' => ['value' => 'In Transit', 'id' => 5], // New
            'receivedbybranch' =>  ['value' => 'Received By Office', 'id' => 5], // New
            'selfpick' =>  ['value' => 'self pickup', 'id' => 7],
            'logsheetconfirm' => ['value' => 'Logsheet confirm', 'id' => 5],


        ];

        $status_name = @$statuses[$status];




        return ["status" => $status, "status_name" => $status_name];
    }

    public function shipmentsInOut(Request $request)
    {

        $request->validate([
            "key" => "required",
            "offset" => "required|integer",
            "limit" => "required|integer",
            "store_username" => "required|exists:login,s_username",
            "tab" => "nullable|in:in,out",
            "date_range" => "nullable|array"
        ]);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }

        try {

            $store_username = $request->store_username;
            $module = $request->module;


            $total_requests = 0;
            $requests = null;

            $offset = $request->offset;
            $limit = $request->limit;

            //status return=in, else out
            $tab = $request->tab;


            $locations = $request->locations ? collect(json_decode($request->locations)) : Null;
            $statuses = $request->statuses ? collect(json_decode($request->statuses, true)) : NULL;
            $date_range = $request->date_range ? json_decode($request->date_range, true) : [];
            $search = $request->search ? $request->search : NULL;



            $total_requests = AppRequest::where("store_id", $store_username)
                ->when($tab == "in", function ($q) {
                    $q->where("r_status", "return");
                })->count();

            $requests = AppRequest::leftJoin("collection as col", "requests.co_id", "col.co_id")
                ->leftJoin("pre_invoice_map as pim", "requests.r_id", "pim.actual_id")
                ->where("requests.store_id", $store_username)
                ->when($tab == "in", function ($q) {
                    return $q->where("requests.r_status", "return");
                })
                ->when($tab != "in", function ($q) {
                    return $q->where("requests.r_status", "!=", "return");
                })
                ->when($locations, function ($query, $locations) {

                    return $query->whereIn("requests.r_wallayah", $locations->pluck("wilaya_id"));
                })
                ->when($date_range, function ($query, $date_range) {

                    return $query->whereBetween("requests.created_at", $date_range);
                })
                ->when($statuses, function ($query, $statuses) {

                    return $query->whereIn("requests.r_status", $statuses->pluck("status"));
                })
                ->when($search, function ($query, $search) {
                })
                ->select(
                    "requests.r_id as order_id",
                    DB::raw('CONCAT("P",pim.pid) as order_no'),
                    "requests.co_id as ref_id",
                    "col.collected_cash as cop",
                    "requests.r_phone as customer_no",
                    "requests.r_name as customer_name",
                    "requests.cc as cc",
                    "requests.r_status as order_status_name",
                    "requests.r_status as order_status_key",
                    "requests.pickup_image as order_pickup_image",
                    "requests.order_image as order_deliver_image",
                    "requests.undeliver_image as order_undeliver_image",
                    "requests.weight as weight",
                    "requests.r_phone as phone",
                    "requests.r_ship_price as shipping_price",
                    "requests.r_price as cod",
                    "requests.external_way_bill_number",
                    "requests.created_at as created_at",
                    "requests.updated_at as updated_at"
                )
                ->orderBy("requests.updated_at", "desc")
                ->skip($offset)->take($limit)->get();



            $resp = $this->processOrders($total_requests, $requests);

            return response()->json(["success" => "ok", "data" => [
                "total_shipments" => $resp["total_requests"],
                "shipments" => $resp["data"],
            ]], 200);
        } catch (\Exception $e) {
            return response()->json([
                "status" => "error",
                "message" => $e->getMessage(),
                "trace" => $e->getTraceAsString(),
            ], 422);
        }
    }



    public function traderPickup(Request $request)
    {
        $request->validate([
            "key" => "required",
            "offset" => "required|integer",
            "limit" => "required|integer",
            "store_username" => "required|exists:login,s_username",
            "tab" => "nullable|in:today,all",
        ]);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }

        $offset = $request->offset;
        $limit = $request->limit;

        $store_username = $request->store_username;
        $tab = $request->tab;


        $total_references = DB::table("collection as col")
            ->where("col.store_id", $store_username)
            ->when($tab == "today", function ($q) {
                $q->whereDate('col.create_time', date("Y-m-d"));
            })
            ->count();


        $collections = DB::table("collection as col")
            ->leftJoin("requests as rq", "col.co_id", "rq.co_id")
            ->leftJoin("wilaya as w", "col.wallayah_id", "w.id")
            ->leftJoin("regions as rg", "col.region_id", "rg.re_id")
            ->leftJoin("login as dl", "col.driver_id", "dl.s_id")
            ->where("col.store_id", $store_username)
            ->when($tab == "today", function ($q) {
                $q->whereDate('col.create_time', date("Y-m-d"));
            })
            ->select(
                "col.co_id as id",
                "col.collected_cash as cop",
                DB::raw("CONCAT(w.name_en,', ',rg.re_name) as location"),
                DB::raw("COUNT(rq.r_id) as total_orders"),
                DB::raw("DATE_FORMAT(col.create_time, '%Y-%m-%d') as collection_date"),
                // "col.create_time as collection_date",
                DB::raw("(CASE WHEN col.is_completed=1 then 'Being Processed' else 'Complete' END) as status"),
                "dl.s_name as driver_name",
                "dl.s_mobile as driver_mobile",
                "col.store_id as store_username",
                DB::raw("(CASE WHEN LENGTH(dl.s_mobile)>=11 then dl.s_mobile else CONCAT('+968',dl.s_mobile) END) as driver_mobile")
            )
            ->groupBy("col.co_id")
            ->orderBy("col.create_time", "desc")
            ->skip($offset)->take($limit)->get();


        return response()->json(["success" => "ok", "data" => ["total_references" => $total_references, "references" => $collections]], 200);
    }


    public function openFinance(Request $request)
    {

        $request->validate([
            "key" => "required",
            "store_username" => "required|exists:login,s_username",
        ]);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }
        $store_username = $request->store_username;

        $requests = AppRequest::where("store_id", $store_username)
            ->where("is_paid", 1)
            ->whereNull("r_closing_date")
            ->where("r_status", "completed")
            ->get();

        $remaining = $requests->sum("price") - $requests->sum("ship_price") - $requests->sum("cc");

        return response()->json([
            "success" => "ok",
            "data" => [
                "remaining" => $remaining,
                "total_orders_delivered" => $requests->count(),
                "total_amount_request" => $requests->sum("price"),
                "delivery_fee" => $requests->sum("ship_price"),
                "collection_fee" => $requests->sum("cc")
            ]
        ], 200);
    }

    public function closeFinance(Request $request)
    {

        $request->validate([
            "key" => "required",
            "store_username" => "required|exists:login,s_username",
            "offset" => "required|integer|min:0",
            "limit" => "required|integer",
        ]);


        if ($request->key != $this->key) {
            return response()->json([
                "status" => "error",
                "message" => "Invalid key"
            ], 422);
        }

        $offset = $request->offset;
        $limit = $request->limit;


        $store_username = $request->store_username;

        $store = DB::table("login")->where("s_username", $store_username)->first();

        $total_invoices = DB::table("closing_requests as ci")
            ->where("ci.store_id", $store->s_id)
            ->count();


        $invoices = DB::table("closing_requests as ci")
            ->where("ci.store_id", $store->s_id)
            ->select(
                "ci.id as id",
                "ci.request_date as closing_date",
                "ci.cod as cod",
                "ci.shipping as shipping_cost",
                "ci.no_orders as total_orders",
                "ci.cc as cc",
                DB::raw("((ci.cod)-(ci.shipping)-(ci.cc)) as amount_transferred"),
                "ci.created_at as created_at,"
            )
            ->groupBy("ci.id")
            ->orderBy("ci.created_at", "desc")
            ->skip($offset)
            ->take($limit)
            ->get();

        // $remaining = $invoice->requests->sum("price") - $requests->sum("ship_price") - $requests->sum("cc");

        // $invoices = $store->invoices->map(function ($invoice) {
        //     return [
        //         "id" => $invoice->id,
        //         "closing_date" => Auth::user()->dateFormat($invoice->close_date),
        //         "cod" => $invoice->cod,
        //         "shipping_cost" => $invoice->ship_price,
        //         "total_orders" => $invoice->total_orders,
        //         "cc" => $invoice->cc,
        //         "amount_transferred" => AppRequest::where("invoice_id", $invoice->id)->sum("price") - AppRequest::where("invoice_id", $invoice->id)->sum("ship_price") - AppRequest::where("invoice_id", $invoice->id)->sum("cc"),
        //         "created_at" => $invoice->created_at->format("d M, Y"),
        //     ];
        // });



        return response()->json([
            "success" => "ok",
            "data" => [
                "total_invoices" => $total_invoices,
                "invoices" => $invoices
            ]
        ], 200);
    }


    public function collectionOrders(Request $request)
    {
        $request->validate([
            "collection_id" => "required|exists:collection,co_id",
        ]);





        $orders = DB::table("requests as rq")
            ->leftJoin("pre_invoice_map as pim", "rq.r_id", "pim.actual_id")
            ->leftJoin("wilaya as w", "rq.r_wallayah", "w.id")
            ->leftJoin("regions as rg", "rq.r_region", "rg.re_id")
            ->where("rq.co_id", $request->collection_id)

            ->select(
                "rq.co_id as ref_id",
                "rq.created_at as created_at",
                "rq.r_id as order_id",
                DB::raw('CONCAT("P",pim.pid) as order_no'),
                DB::raw("CONCAT(w.name_en,', ',rg.re_name) as location"),
                "rq.r_phone as phone",
                "rq.r_status as order_status_name",
                "rq.r_status as order_status_key",
                "rq.pickup_image as order_pickup_image",
                "rq.order_image as order_deliver_image",
                "rq.undeliver_image as order_undeliver_image",
                "rq.weight as weight",
                "rq.r_ship_price as shipping",
                "rq.r_price as cod",
                "rq.updated_at as updated_at",
                "rq.external_way_bill_number"
            )
            ->orderBy("rq.updated_at", "desc")
            ->get();



        if ($orders) {
            $orders = $orders->map(function ($order) {

                $image = $this->getOrderImagesDalilee(["image_name" => $order->order_pickup_image, "created_at" => $order->created_at]);

                if ($order->order_no == "" || is_null($order->order_no)) {
                    $order->order_no = $order->external_way_bill_number ? $order->external_way_bill_number : "A" . $order->order_id;
                }
                return [
                    "ref_id" => $order->ref_id,
                    "date" => $order->created_at,
                    "order_id" => $order->order_id,
                    "order_no" =>  $order->order_no,
                    "image" => $image,
                    "weight" => $order->weight,
                    "shipping" => $order->shipping,
                    "phone" => $order->phone,
                    "location" => $order->location
                ];
            });
        } else {
            $orders = [];
        }


        return response()->json(["success" => "ok", "data" => ["orders" => $orders]], 200);
    }
}
